package DDW.Vex2D
{
	import Box2D.Collision.*;
	import Box2D.Collision.Shapes.*;
	import Box2D.Common.*;
	import Box2D.Common.Math.*;
	import Box2D.Dynamics.*;
	import Box2D.Dynamics.Contacts.*;
	import Box2D.Dynamics.Joints.*;
	
	import flash.display.Graphics;
	import flash.display.Sprite;
	import flash.geom.ColorTransform;
	import flash.geom.Point;
	
	public class SceneCreator extends World
	{               
		private var bodies:Array = [];
		private var bodyMap:Object = {};
		private var movieclips:Array = [];		
		private var movieclipMap:Object = {};		
		private var joints:Array = [];
		
		private var drawLayer:int;
		private var drawSprite:Sprite;
        
		public function SceneCreator(data:SceneData)
		{
			super(data);
			bodyMap["_ground"] = m_world.m_groundBody;
		}	
		
		public function CreateFullScene():void
		{
			for(var i:int = 0; i < data.instances.length; i++)
			{
				AddInstance(data.instances[i]);
			}
			
			for(var j:int = 0; j < data.joints.length; j++)
			{
				AddJoint(data.joints[j]);
			}
			
			drawSprite = new Sprite();
			Main.rootSprite.addChildAt(drawSprite, drawLayer);
		}
		public function CreateConnectedSet(instance:String, location:Point):void
		{
			
		}
		
		public  function AddInstance(instance:Object):void
		{	
			var x:Number = instance.location.x / m_physScale;
			var y:Number = instance.location.y / m_physScale;
			var angle:Number = (instance.rotation == null) ? 0 : instance.rotation;
            var sx:Number = (instance.scaleX != null) ? instance.scaleX : 1;
            var sy:Number = (instance.scaleY != null) ? instance.scaleY : 1;
	        var shapes:Array = data.data[instance.type];
			
			// sprite
        	var bx:Sprite = new data[data.mapping[instance.type]];
        	if(sx != 1)
        	{
        		bx.scaleX = sx;
        	}
        	if(sy != 1)
        	{
        		bx.scaleY = sy;
        	}
        	if(instance.colorTransform != null)
        	{
        		var c:Object = instance.colorTransform;
        		var ct:ColorTransform = new ColorTransform(c.rb/255, c.gb/255, c.bb/255, c.ab/255, c.ra, c.ga, c.ba, c.aa);
        		bx.transform.colorTransform = ct;
        	}
            Main.rootSprite.addChild(bx);
            movieclipMap[instance.instName] = bx;
            
            // box2D body
			if(shapes.length > 0)
			{
				var bodyDef:b2BodyDef = new b2BodyDef();
	            bodyDef.position.Set(x, y);
	            bodyDef.angle = angle;
	            
	            var body:b2Body = m_world.CreateBody(bodyDef);
	            bodies.push(body);
	            bodyMap[instance.instName] = body;            
	            
	            for(var i:int = 0; i < shapes.length; i++)
	            {
	            	AddPoly(body, shapes[i], instance, sx, sy);
	            }
	            body.SetMassFromShapes();
	            
           		movieclips.push(bx);
			}
			else
			{
				bx.x = instance.location.x;
				bx.y = instance.location.y;
				bx.rotation = instance.rotation / Math.PI * 180;
				drawLayer = Main.rootSprite.numChildren;
			}	
		}
		public  function AddPoly(body:b2Body, shp:Object, inst:Object, scaleX:Number, scaleY:Number):void
		{				
			var bx:Sprite;
			var sd:b2ShapeDef;
        	if(shp.radius != null)
        	{
				var circDef:b2CircleDef = new b2CircleDef();
				circDef.radius = shp.radius / m_physScale;	
				var lp:b2Vec2 = new b2Vec2();
				lp.Set(shp.center.x / m_physScale, shp.center.y / m_physScale);
				circDef.localPosition = lp;
				sd = circDef;
        	}
        	else
        	{
        		var pts:Array = shp as Array;
				var polyDef:b2PolygonDef = new b2PolygonDef();
				sd = polyDef;			
				polyDef.vertexCount = pts.length / 2;
				
				for(var i:int = 0; i < polyDef.vertexCount; i++)
				{
					polyDef.vertices[i].Set( 
						pts[i*2] / m_physScale * scaleX,  	 
						pts[i*2+1] / m_physScale * scaleY);
				}
        	}
        	
            if(inst.instName.indexOf("s_") == 0)
            {
				sd.density = 0.0;
            }       
            else
            {
            	sd.density = inst["density"] == null ? 2.0 : inst["density"];
            }
            sd.friction = inst["friction"] == null ? 0.3 : inst["friction"];
            sd.restitution = inst["restitution"] == null ? 0.1 : inst["restitution"];
			
			body.CreateShape(sd);
		}
	
		public  function AddJoint(joint:Object):void
		{	
			var targ0:b2Body = bodyMap[joint["body1"]];
			var targ1:b2Body = bodyMap[joint["body2"]];
			var pt0:Object = joint["location"];
			
			var anchor0:b2Vec2 = new b2Vec2();
			anchor0.Set((pt0.x) / m_physScale, (pt0.y) / m_physScale);	
			
			switch(joint.type)
			{			
				case "Distance":
					var pt1:Object = joint["location2"];		
					var anchor1:b2Vec2 = new b2Vec2();
					anchor1.Set((pt1.x) / m_physScale, (pt1.y) / m_physScale);
					
					var dj:b2DistanceJointDef = new b2DistanceJointDef();
					dj.Initialize(targ0, targ1, anchor0, anchor1);
					dj.collideConnected = joint["collideConnected"] == null ? true : joint["collideConnected"];
					if(joint["dampingRatio"] != null)
					{
						dj.dampingRatio = joint["dampingRatio"]; 
					} 
					if(joint["frequencyHz"] != null)
					{
						dj.frequencyHz = joint["frequencyHz"]; 
					} 
					if(joint["length"] != null)
					{
						if(typeof(joint["length"]) == "string")
						{
							dj.length = parseFloat(joint["length"]) / m_physScale;
						}
						else
						{
							dj.length = joint["length"] / m_physScale;
						}
					}
					m_world.CreateJoint(dj);
					joints.push(dj);
					break;
					
				case "Revolute":
					var rot0:Number = (typeof(joint["min"]) == "string") ? parseFloat(joint["min"]) / 180 * Math.PI : joint["min"];
					var rot1:Number = (typeof(joint["max"]) == "string") ? parseFloat(joint["max"]) / 180 * Math.PI : joint["max"];

					var rj:b2RevoluteJointDef = new b2RevoluteJointDef();
					rj.Initialize(targ0, targ1, anchor0);
					rj.lowerAngle = rot0;
					rj.upperAngle = rot1;
					
					rj.enableLimit = joint["enableLimit"] == null ? true : joint["enableLimit"];
					rj.maxMotorTorque = joint["maxMotorTorque"] == null ? 10 : joint["maxMotorTorque"];
					rj.motorSpeed = joint["motorSpeed"] == null ? 0 : joint["motorSpeed"];
					rj.enableMotor = joint["enableMotor"] == null ? true : joint["enableMotor"];
					
					m_world.CreateJoint(rj);
					joints.push(rj);
					break;
					
				case "Prismatic":
					var axisX:Number = joint["axisX"];
					var axisY:Number = joint["axisY"];
					var min:Number = (typeof(joint["min"]) == "string") ? parseFloat(joint["min"]) : joint["min"];
					var max:Number = (typeof(joint["max"]) == "string") ? parseFloat(joint["max"]) : joint["max"];

					var pj:b2PrismaticJointDef = new b2PrismaticJointDef();					
					var worldAxis:b2Vec2 = new b2Vec2();
					worldAxis.Set(axisX, axisY);
					pj.Initialize(targ0, targ1, anchor0, worldAxis);
					pj.lowerTranslation = min / m_physScale;
					pj.upperTranslation = max / m_physScale;
					
					pj.enableLimit = joint["enableLimit"] == null ? true : joint["enableLimit"];
					pj.maxMotorForce = joint["maxMotorForce"] == null ? 10 : joint["maxMotorForce"];
					pj.motorSpeed = joint["motorSpeed"] == null ? 0 : joint["motorSpeed"];
					pj.enableMotor = joint["enableMotor"] == null ? true : joint["enableMotor"];
					
					m_world.CreateJoint(pj);
					joints.push(pj);
					break;
				case "Pully":					
					var pt2:Object = joint["location2"];					
					anchor1 = new b2Vec2();
					anchor1.Set((pt2.x) / m_physScale, (pt2.y) / m_physScale);					
					
					var ga1:Object = joint["groundAnchor1"];
					var groundAnchor0:b2Vec2 = new b2Vec2();
					groundAnchor0.Set((ga1.x) / m_physScale, (ga1.y) / m_physScale);
					
					var ga2:Object = joint["groundAnchor2"];
					var groundAnchor1:b2Vec2 = new b2Vec2();
					groundAnchor1.Set((ga2.x) / m_physScale, (ga2.y) / m_physScale);
					
					var max0:Number = joint["maxLength1"];
					var max1:Number = joint["maxLength2"];
					
					var rat:Number = joint["ratio"] == null ? 1 : joint["ratio"];
					
					var puj:b2PulleyJointDef = new b2PulleyJointDef();
					puj.Initialize(targ0, targ1, groundAnchor0, groundAnchor1, anchor0, anchor1, rat);
					puj.maxLength1 = (max0 + max1) / m_physScale;
					puj.maxLength2 = (max0 + max1) / m_physScale;

					puj.collideConnected = joint["collideConnected"] == null ? true : joint["collideConnected"];

					m_world.CreateJoint(puj);
					joints.push(puj);
					break;
				case "Gear":
					var gj:b2GearJointDef = new b2GearJointDef();
					gj.body1 = targ0;
					gj.body2 = targ1;
					gj.joint1 = getFirstGearableJoint(targ0.m_jointList);
					gj.joint2 = getFirstGearableJoint(targ1.m_jointList);
					gj.ratio = joint["ratio"] == null ? 1 : joint["ratio"];
					m_world.CreateJoint(gj);
					joints.push(gj);
					break;
			}
		}	
		private function getFirstGearableJoint(je:b2JointEdge):b2Joint
		{
			var result:b2Joint = je.joint;
			while(result != null && !(result is b2PrismaticJoint || result is b2RevoluteJoint))
			{
				je = je.next;
				result = je.joint;
			}
			return result;
		}
		
		public override function Update():void
		{
			super.Update();	
			
			for(var i:int = 0; i < movieclips.length; i++)
			{
				var s:Sprite = Sprite(movieclips[i]);
				var b:b2Body = bodies[i];
				var p:b2Vec2 = b.GetPosition();
				movieclips[i].x = bodies[i].GetPosition().x * m_physScale;
				movieclips[i].y = bodies[i].GetPosition().y * m_physScale;
				movieclips[i].rotation = bodies[i].GetAngle()/ Math.PI * 180;
				//s.transform.matrix
			}
			var g:Graphics = this.drawSprite.graphics;	
			g.clear();
			for(i = 0; i < joints.length; i++)
			{
				if(joints[i] is b2PulleyJointDef)
				{
					var puj:b2PulleyJointDef = joints[i] as b2PulleyJointDef;
					g.lineStyle(2, 0x444444);
					g.moveTo(puj.body1.GetPosition().x * m_physScale, puj.body1.GetPosition().y * m_physScale);
					g.lineTo(puj.groundAnchor1.x * m_physScale, puj.groundAnchor1.y * m_physScale);
					g.lineStyle(5, 0x221133);
					g.lineTo(puj.groundAnchor2.x * m_physScale, puj.groundAnchor2.y * m_physScale);
					g.lineStyle(2, 0x444444);
					g.lineTo(puj.body2.GetPosition().x * m_physScale, puj.body2.GetPosition().y * m_physScale);
				}
			}
			
		}

		
	}
}